/*

Copyright 1992 by the University of Edinburgh, Department of Computer Science

Permission to use, copy, modify, distribute, and sell this software and its
documentation for any purpose is hereby granted without fee, provided that
the above copyright notice appear in all copies and that both that
copyright notice and this permission notice appear in supporting
documentation, and that the name of the University of Edinburgh not be used
in advertising or publicity pertaining to distribution of the software
without specific, written prior permission.  The University of Edinburgh
makes no representations about the suitability of this software for any
purpose.  It is provided "as is" without express or implied warranty.

*/
/* This little program simply forks itself, with the parent doing nothing more
 * than exiting.  The child closes its stdin, stdout and stderr, and then
 * reopens them to point to /dev/null and a log file.  The motivation is
 * to get rid of all the junk processes that get left lying around when rsh
 * is used to start up remote X clients.
 */

/* 15/2/90 Modified to look at Environment variable XLOG for name of log
   file. (PA) */

#include <stdio.h>
#include <string.h>
#include <fcntl.h>

#define MAXARGS 128
#define LOGFILE "/.XLog"
#define LOGMAX 256

char *findenv();
char *getenv();

main(argc, argv, envp)
int argc;
char **argv, **envp;
{	int pid;
	char *path, *name;
	char logbufr[LOGMAX];
	char *args[MAXARGS];
	char *logfile;
	int n, nds;
	if (--argc == 0) {
		fprintf(stderr, "Xexec: no program specified\n");
		exit(1);
	}
	*argv++;
	if ((pid = fork()) < 0) {
		perror("Xexec: fork");
		exit(1);
	}
	if (pid != 0) _exit(0);	/* parent */
#if defined(hpux) || defined(Solaris)
	nds = 32;  /* Hack */
#else
	nds = getdtablesize();
	if (nds < 0) {
		perror("getdtablesize");
		exit(1);
	}
#endif
	for (n = 3; n < nds; n++) (void) close(n);
	if ((n = open("/dev/null", O_RDONLY)) < 0) {
		perror("Xexec: open /dev/null");
		exit(1);
	}
	(void) dup2(n, 0);  (void) close(n);
	path = findenv("HOME=", envp);
	if (path == NULL) {
		fprintf(stderr, "Can't find $HOME\n");
		exit(1);
	}
	path = strchr(path, '=');
	if (path == NULL) {
		fprintf(stderr, "$HOME corrupt\n");
		exit(1);
	}
	if ((logfile=getenv("XLOG"))==NULL) {
	  (void) strcpy(logbufr, path+1);
	  (void) strcat(logbufr, LOGFILE);
	  logfile=logbufr;
	}
	if ((n = open(logfile, O_WRONLY | O_APPEND | O_CREAT, 0644)) < 0) {
		perror("Xexec: open logfile");
		exit(1);
	}
	(void) dup2(n, 1);  (void) dup2(n, 2);  (void) close(n);
	path = *argv++;  argc--;
	if ((args[0] = strrchr(path, '/')) == NULL) args[0] = path;
	else args[0]++;
	n = 1;
	while (argc-- > 0) args[n++] = *argv++;
	args[n] = NULL;
	(void) execvp(path, args);
	/* If we get here the exec has failed */
	perror("Xexec: execvp");
	exit(0);
}

static char *findenv(what, envp)
char *what;
char **envp;
{       int     n;
        n = strlen(what);
        while (*envp != NULL) {
                if (!strncmp(what, *envp, n)) return *envp;
                *envp++;
        }
        return NULL;
}
