/*

Copyright 1992 by the University of Edinburgh, Department of Computer Science

Permission to use, copy, modify, distribute, and sell this software and its
documentation for any purpose is hereby granted without fee, provided that
the above copyright notice appear in all copies and that both that
copyright notice and this permission notice appear in supporting
documentation, and that the name of the University of Edinburgh not be used
in advertising or publicity pertaining to distribution of the software
without specific, written prior permission.  The University of Edinburgh
makes no representations about the suitability of this software for any
purpose.  It is provided "as is" without express or implied warranty.

*/
/* Probe Sun frame buffers.  Write one of "mono" or "colour" to stdout, 
 * possibly qualified with "multi-head".  Just wander down a list of
 * devices, trying to open each one in turn and tallying the results
 */

#include <fcntl.h>
#include <stdio.h>
#include <sys/ioccom.h>
#ifdef Solaris
#include <sys/fbio.h>
#else
#include <sun/fbio.h>
#endif

static char *mono[] = { "bwtwo", NULL };
static char *colour[] = { "cgthree", "cgfour", "cgsix", "cgeight",
			"cgnine", "cgtwelve", "cgtwo", NULL };

static int probe();

static int verbose = 0;

main(argc, argv)
int argc;
char **argv;
{	int n, total = 0, col = 0;
	char **type;

	if (argc > 1 && !strcmp(argv[1], "-v")) verbose = 1;

	type = mono;
	while (*type) {
		n = probe(*type);
		if (n) total += n;
		type++;
	}

	type = colour;
	while (*type) {
		n = probe(*type);
		if (n) {
			total += n;
			col++;
		}
		type++;
	}

	(void) printf(total > 1 ? "multiHead-%s\n" : "%s\n",
			col ? "colour" : "mono");
}

static int probe(device)
char *device;
{	char buffer[32];
	int i, fd, status;
	int tally = 0;
	struct fbtype info;
	struct fbgattr attr;

#ifdef DEBUG
	(void) fprintf(stderr, "probe %s\n", device);
#endif

	for (i = 0;  i < 3; i++) {
		sprintf(buffer, "/dev/%s%d", device, i);
		fd = open(buffer, O_RDONLY, 0);
#ifdef DEBUG
		(void) fprintf(stderr, "%s status %d\n", buffer, fd);
#endif
		if (verbose && (fd > 0)) {
			status = ioctl(fd, FBIOGTYPE, &info);
			if (status >= 0)
				(void) printf("%s type %d, width %d, height %d\n",
					buffer, info.fb_type,
					info.fb_width, info.fb_height);
			else perror(device);
		}

		if (verbose && (fd > 0)) {
			status = ioctl(fd, FBIOGATTR, &attr);
			if (status >= 0)
				(void) printf("attr: %s real type %d type %d width %d height %d\n",
					buffer, attr.real_type,
					attr.fbtype.fb_type,
					attr.fbtype.fb_width,
					attr.fbtype.fb_height);
			else perror(device);
		}

		if (fd > 0) {
			tally++;
			(void) close(fd);
		}
	}
	return tally;
}
