/*

Copyright 1992 by the University of Edinburgh, Department of Computer Science

Permission to use, copy, modify, distribute, and sell this software and its
documentation for any purpose is hereby granted without fee, provided that
the above copyright notice appear in all copies and that both that
copyright notice and this permission notice appear in supporting
documentation, and that the name of the University of Edinburgh not be used
in advertising or publicity pertaining to distribution of the software
without specific, written prior permission.  The University of Edinburgh
makes no representations about the suitability of this software for any
purpose.  It is provided "as is" without express or implied warranty.

*/
/* This little utility is run from a user's xdm login session.  It opens a log
   file, then forks the favourite login shell (thereby implicitly setting up
   the correct environment) to run the Xsession script. */

#include <stdio.h>
#include <string.h>
#include <fcntl.h>

#define SESSION "/usr/local/share/X11R5/xdm/Xsession"
#define LOGFILE "/.xdm-log"
#define LOGMAX 256

static char *failsafeShell = "/usr/local/bin/X11R5/xterm";
static char *failsafeArgs[] =
	{ "xterm", "-geometry", "80x24+0-0", "-ls" };

char *getenv();

main(argc, argv, envp)
int argc;
char **argv, **envp;
{	char *path, *name;
	char logbufr[LOGMAX];
	char *logfile;
	char *shell, *shellname;
	char arg0[LOGMAX];
	char *args[3];
	int n;
	if ((n = open(SESSION, O_RDONLY)) < 0) {
		perror("xdm-session: open session file");
		exit(1);
	}
	(void) dup2(n, 0);  (void) close(n);
	path = getenv("HOME");
	if (path == NULL) {
		fprintf(stderr, "$HOME corrupt\n");
		exit(1);
	}
	if ((logfile = getenv("XDMLOG")) == NULL) {
		(void) strcpy(logbufr, path);
		(void) strcat(logbufr, LOGFILE);
		logfile = logbufr;
	}
	if ((n = open(logfile, O_WRONLY | O_CREAT | O_APPEND, 0644)) < 0) {
		perror("xdm-session: open logfile");
		exit(1);
	}
	(void) dup2(n, 1);  (void) dup2(n, 2);  (void) close(n);
	if (argc == 2 && !strcmp(argv[1], "failsafe")) {
		(void) execvp(failsafeShell, failsafeArgs);
		perror("xdm-session: execvp failsafe");
	}
	shell = getenv("SHELL");
	if (shell == NULL) {
		(void) fprintf(stderr, "xdm-session: no $SHELL\n");
		exit(1);
	}
	if ((shellname = strrchr(shell, '/')) == NULL) shellname = shell;
	else shellname ++;
	*arg0 = '-';
	strcpy(arg0 + 1, shellname);
	args[0] = arg0;
	args[1] = NULL;
	args[2] = NULL;
	(void) execvp(shell, args);
	/* If we get here the exec has failed */
	perror("xdm-session: execvp");
	exit(2);
}
